@@ -356,7 +356,7 @@ class ComparePreviews(GenericRunAction):
         return True
     
     def extra_argv(self):
-        return ['-wipe'] + self._previews
+        return ['-wipe', '-autoRetime', '0'] + self._previews
     
     def child_value_changed(self, child_value):
         if child_value is self.task_1:
@@ -593,6 +593,8 @@ class PlayblastItem(flow.Object):
 
     to_submit = flow.SessionParam(False).ui(editor='bool').watched()
     use_simplify = flow.SessionParam(False).ui(editor='bool')
+    reduce_textures = flow.SessionParam(False).ui(editor='bool')
+    target_texture_width = flow.SessionParam(False).ui(editor='bool')
     priority = flow.SessionParam(10).ui(editor='int').watched()
 
     def child_value_changed(self, child_value):
@@ -693,6 +695,8 @@ class PlayblastFile(PlayblastItem):
 
         file_object.submit_blender_playblast_job.revision_name.set(self.revision.get())
         file_object.submit_blender_playblast_job.use_simplify.set(self.use_simplify.get())
+        file_object.submit_blender_playblast_job.reduce_textures.set(self.reduce_textures.get())
+        file_object.submit_blender_playblast_job.target_texture_width.set(self.target_texture_width.get())
         file_object.submit_blender_playblast_job.priority.set(self.priority.get())
         file_object.submit_blender_playblast_job.pool.set(pool_name)
         file_object.submit_blender_playblast_job.run('Add to render pool')
@@ -936,13 +940,15 @@ class RenderSequencePlayblasts(ProcessSequenceFiles):
         site = self.root().project().get_current_site()
         return ['default'] + site.pool_names.get()
     
-    def submit_blender_playblast_job(self, shot_name, dept_name, file_name, revision_name, use_simplify, priority=10, pool_name='default'):
+    def submit_blender_playblast_job(self, shot_name, dept_name, file_name, revision_name, use_simplify, reduce_textures, target_texture_width, priority=10, pool_name='default'):
         file_name = file_name.replace('.', '_')
         oid = f'{self._sequence.oid()}/shots/{shot_name}/departments/{dept_name}/files/{file_name}'
         file = self.root().get_object(oid)
         
         file.submit_blender_playblast_job.revision_name.set(revision_name)
         file.submit_blender_playblast_job.use_simplify.set(use_simplify)
+        file.submit_blender_playblast_job.reduce_textures.set(reduce_textures)
+        file.submit_blender_playblast_job.target_texture_width.set(target_texture_width)
         file.submit_blender_playblast_job.priority.set(priority)
         file.submit_blender_playblast_job.pool.set(pool_name)
         file.submit_blender_playblast_job.run('Submit job')
