@@ -45,6 +45,73 @@ class JobStatus(flow.values.ChoiceValue):
     CHOICES = ['WFA', 'WAITING', 'PROCESSED', 'ERROR', 'PAUSE', 'DONE']
 
 
+class ResetJob(flow.Action):
+
+    ICON = ('icons.libreflow', 'reset')
+
+    _job = flow.Parent()
+    _jobs = flow.Parent(2)
+
+    def needs_dialog(self):
+        o = self.root().get_object(self._job.emitter_oid.get())
+        return self._job.type.get() == 'Upload' and o.get_sync_status(exchange=True) == 'Available'
+    
+    def allow_context(self, context):
+        return context and self._job.status.get() == 'ERROR'
+    
+    def get_buttons(self):
+        self.message.set((
+            '<h3>Revision already available on exchange server</h3>'
+            'Reset upload job anyway ?'
+        ))
+        return ['Confirm', 'Cancel']
+
+    def run(self, button):
+        if button == 'Cancel':
+            return
+        
+        self._job.status.set('WAITING')
+        self._job.log.set('?')
+        self._jobs.incr_waiting_count()
+        o = self.root().get_object(self._job.emitter_oid.get())
+        o.set_sync_status('NotAvailable', exchange=True)
+
+
+class ResetJobs(flow.Action):
+
+    ICON = ('icons.libreflow', 'reset')
+
+    _jobs = flow.Parent()
+
+    def needs_dialog(self):
+        return False
+
+    def run(self, button):
+        if button == 'Cancel':
+            return
+        
+        for j in self._jobs.jobs(status='ERROR'):
+            j.status.set('WAITING')
+        
+        self._jobs.touch()
+
+
+class ShowRevisionInHistory(flow.Action):
+
+    ICON = ("icons.gui", "ui-layout")
+
+    _job = flow.Parent()
+
+    def needs_dialog(self):
+        return False
+
+    def run(self, button):
+        history_oid = self._job.emitter_oid.get() + '/../..'
+        history_oid = self.root().session().cmds.Flow.resolve_path(history_oid)
+
+        return self.get_result(goto=history_oid)
+
+
 class Job(flow.Object):
 
     type = flow.Param('Download', LoadType)
@@ -58,6 +125,9 @@ class Job(flow.Object):
     requested_by_studio = flow.Param()
     log = flow.Param().ui(editor='textarea')
 
+    reset = flow.Child(ResetJob)
+    show = flow.Child(ShowRevisionInHistory).ui(label='Show in history')
+
     def __repr__(self):
         job_repr = "%s(type=%s, status=%s, priority=%s, emitter=%s, date=%s, date_end=%s, is_archived=%s, requested_by_user=%s, requested_by_studio=%s)" % (
                 self.__class__.__name__,
@@ -78,6 +148,7 @@ class Job(flow.Object):
 class JobQueue(flow.Map):
 
     count_waiting = flow.IntParam(0)
+    reset_jobs = flow.Child(ResetJobs).ui(label='Reset erroneous jobs')
 
     @classmethod
     def mapped_type(cls):
@@ -162,18 +233,16 @@ class JobQueue(flow.Map):
         return [
             "Type",
             "Status",
-            "Priority",
+            "Revision",
             "Emitted on",
-            "Expires on",
             "Requested by (site/user)",
         ]
     
     def _fill_row_cells(self, row, item):
         row["Type"] = item.type.get()
         row["Status"] = item.status.get()
-        row["Priority"] = item.priority.get()
+        row["Revision"] = item.emitter_oid.get()
         row["Emitted on"] = datetime.fromtimestamp(item.date.get()).ctime()
-        row["Expires on"] = "Never" if item.date_end.get() == -1 else item.date_end.get()
         row["Requested by (site/user)"] = "%s/%s" % (
             item.requested_by_studio.get(),
             item.requested_by_user.get()
@@ -536,6 +605,10 @@ class RequestAs(flow.Action):
     sites = flow.Child(SiteSelection).ui(expanded=True)
     priority = flow.IntParam(50)
     include_dependencies = flow.SessionParam(True).ui(editor='bool')
+    forced_upload = flow.SessionParam(False).ui(
+        editor='bool',
+        tooltip=('Ask the source site to upload the revision regardless'
+                 ' of its status on the exchange site.'))
     
     def get_buttons(self):
         self.sites.source_site.set(self.root().project().get_current_site().name())
@@ -605,7 +678,7 @@ class RequestAs(flow.Action):
         exchange_site = self.root().project().get_exchange_site()
         revision_status = self._revision.get_sync_status(exchange=True)
 
-        if revision_status == "Available":
+        if revision_status == "Available" and not self.forced_upload.get():
             self.root().session().log_warning(
                 "Revision already available on exchange server"
             )
@@ -974,7 +1047,10 @@ class ConfigureWorkingSite(ConfigureSite):
 
 class Queue(flow.Object):
 
-    job_list = flow.Child(JobQueue).ui(label="Jobs", expanded=True)
+    job_list = flow.Child(JobQueue).ui(
+        label="Jobs",
+        expanded=True,
+        show_filter=True)
 
 
 class GotoQueue(flow.Action):
@@ -1167,6 +1243,7 @@ class WorkingSite(Site):
         )
     )
     is_kitsu_admin = flow.BoolParam(False)
+    auto_upload_kitsu_playblasts = flow.BoolParam(True)
 
     root_folder = flow.Computed(cached=True)
     root_windows_folder = flow.Param()
@@ -1706,12 +1783,22 @@ class UploadRevision(flow.Action):
     _revisions = flow.Parent(2)
     
     def needs_dialog(self):
-        return False
+        return self._revision.get_sync_status(exchange=True) == 'Available'
     
     def allow_context(self, context):
         return context and not self._revision.is_working_copy()
     
+    def get_buttons(self):  
+        self.message.set((
+            '<h3>Revision already available on exchange server</h3>'
+            'Upload and overwrite it anyway ?'
+        ))
+        return ['Confirm', 'Cancel']
+    
     def run(self, button):
+        if button == 'Cancel':
+            return
+
         current_site = self.root().project().get_current_site()
         # Add an upload job for the current site
         job = current_site.get_queue().submit_job(
@@ -1725,3 +1812,34 @@ class UploadRevision(flow.Action):
         process_jobs_action._process(job)
         
         self._revisions.touch()
+
+
+class DownloadRevision(flow.Action):
+    
+    _revision = flow.Parent()
+    _revisions = flow.Parent(2)
+
+    def needs_dialog(self):
+        return False
+    
+    def allow_context(self, context):
+        return context and self._revision.get_sync_status() != 'Available'
+    
+    def run(self, button):
+        if button == 'Cancel':
+            return
+
+        current_site = self.root().project().get_current_site()
+        # Add an upload job for the current site
+        job = current_site.get_queue().submit_job(
+            job_type='Download',
+            init_status='WAITING',
+            emitter_oid=self._revision.oid(),
+            user=self.root().project().get_user(),
+            studio=current_site.name(),
+        )
+        self._revision.set_sync_status('Requested', current_site.name())
+        process_jobs_action = self.root().project().admin.process_jobs
+        process_jobs_action._process(job)
+        
+        self._revisions.touch()
